#ifndef __MACRO_H__
#define __MACRO_H__
#include <iostream>
#include <string>
#include <vector>
using namespace std;
/** @addtogroup main */
// @{ 

/// swap choice is set randomly to 0 or 1 depending on the "position" / "seed shape" being modified ...
extern int                    gv_swap_choice;
/// flag to select symetric seeds only
extern bool                   gv_symetric;
/// number of runs (0 when full enumeration)
extern int                    gv_nbruns;
/// flag that enables the "overdubbed" symbols
extern bool                   gv_matching_symbol_flag;
/// length of the alignment on which seeds are optimized
extern int                    gv_alignment_length;
/// flag that enables the "subalignment" computation
extern bool                   gv_subalignment_flag;
/// length of the "subalignment" 
extern int                    gv_subalignment_length;
/// alignment alphabet size
extern int                    gv_align_alphabet_size;
/// seed alphabet size
extern int                    gv_seed_alphabet_size;
/// signature vector (number of seed letters per seed)
extern vector<int>            gv_signature;
/// signature flag that enable the signature selection
/// @see gv_signature
extern bool                   gv_signature_flag;
/// weight for each seed letter
extern vector<double>         gv_bsel_weight;

/// min seed letter selectivity
extern double                 gv_bsel_minprob;
/// max seed letter selectivity
extern double                 gv_bsel_maxprob;

/// min seed span
extern int                    gv_minspan;
/// max seed span
extern int                    gv_maxspan;

/// min seed weight
extern double                 gv_minweight;
/// max seed weight
extern double                 gv_maxweight;

/// flag that enables min and max seed weight
/// @see gv_minweight,gv_maxweight
extern bool                   gv_weight_interval;
/// flag that enables vectorized seeds
/// @see gv_vectorizedsubsetseed_scoring_matrix,gv_vectorizedsubsetseed_scoring_threshold
extern bool                   gv_vectorized_flag;
/// set of symbols when "overdubbed"
/// @see gv_bsymbols_flag
extern char *                 gv_bsymbols_array;
/// flag that enables "overdubbed" symbols
/// @see gv_bsymbols_array
extern bool                   gv_bsymbols_flag;


/// the lossless cost for A when lossless flag is activated
/// @see gv_lossless_flag
/// @see gv_lossless_cost_threshold
extern vector<int>            gv_lossless_costs_vector;

/** @name sleeping proccess activation
 *  @brief the iedera process goes into idle mode when number of process around is over  gv_nb_processes, or when its time to sleep.
 */
// @{
/// flag that activates possible sleep of the iedera process
extern bool     gv_sleeping_flag;
/// flag that activate the sleep when number of process is over gv_nb_processes
extern bool     gv_nb_processes_flag;
/// set the number of process needed to activate the sleep
extern int      gv_nb_processes;
// @}

 
/*
 * (I) Usefull functions
 */

/// ERROR macro
#define _ERROR(message1,message2) {                             \
  cerr << "*ERROR : " << message1 << " : " << message2 << endl; \
  exit(-1);                                                     \
}

/// WARNING macro
#define _WARNING(message1,message2) {                             \
  cerr << "*WARNING : " << message1 << " : " << message2 << endl; \
}
/// MAX macro
#define MAX(a,b) ((a)>(b)?(a):(b)) 
/// MIN macro
#define MIN(a,b) ((a)<(b)?(a):(b))
/// INFINITY macro for signed int
#define INT_INFINITY   (0x7fffffff)  

/** qpow functions
 */
// @{

/** 
 *@brief quick power on integer base
 *@param x is the base of the power
 *@param n is the exponent of the power
 *@return \f$ x^n \f$
 */
int power(int x, int n);

/** 
 *@brief quick power on double base
 *@param x is the base of the power
 *@param n is the exponent of the power
 *@return \f$ x^n \f$
 */
double power(double x, int n);

// @}


/* debug purposes */
/// some assertions that may be not checked to speedup the process
//#define ASSERTB
/// some build debug 
//#define BUILD

//#define USEQUEUEAUTOMATON // USED BY NOW ??

/* map table or direct index table ? (no hash by now ...) */
///
//#define USEMAPPRODUCT
///
//#define USEMAPMHIT
///
//#define NOMATRIX

/// use a queue for the product (otherwise it is a stack that disables some optimizations) 
#define USEQUEUEPRODUCT
/// use a queue for the mhit automaton (otherwise is is a stack)
#define USEQUEUEMHIT

/// number of runs when "sleeping tests" are done 
#define CMOD 1
/// number of seconds when "sleeping tests" are redone when sleeping
#define CSLEEP 600

/// macro that put the process on sleep for CSLEEP time each CMOD nbruns ...
#define CHECK_PROCESS(MOD) {                                    \
if ((MOD <= 2) || (!(nbruns % (MOD)))) {                        \
  bool sleeping = false;                                        \
 restart:                                                       \
                                                                \
  if(gv_sleeping_flag) {                                        \
                                                                \
    time_t    t    = time(NULL);                                \
    struct tm * tm = localtime(&t);                             \
                                                                \
    if (                                                        \
        (tm->tm_hour >= 8 && tm->tm_hour < 20) &&               \
        (tm->tm_wday > 0  && tm->tm_wday < 6)                   \
        ) {                                                     \
      if(!sleeping) {                                           \
        /* erase automata */                                    \
        for (int i=0; i < gv_nbseeds; i ++) {                   \
           if (a_s[i] != NULL){                                 \
             delete a_s[i];                                     \
             a_s[i] = NULL;                                     \
           }                                                    \
        }                                                       \
                                                                \
        /* security input */                                    \
        if(gv_nb_input_filenames > 0) {                         \
          for(int i=0;i<gv_nb_input_filenames;i++)              \
            inputPareto(l,gv_input_filenames[i]);               \
        }                                                       \
                                                                \
        /* final output */                                      \
        cerr << "# seeds\t sel\t sens\t dist"<<  endl;          \
        selectPareto(l);                                        \
        if (!gv_output_filename)                                \
          list_and_areaPareto(l);                               \
        else                                                    \
          outputPareto(l,gv_output_filename);                   \
        cerr << "sleeping ..." << endl;                         \
                                                                \
        sleeping = true;                                        \
                                                                \
      }                                                         \
      SLEEP(CSLEEP);                                            \
      goto restart;                                             \
                                                                \
    } else {                                                    \
                                                                \
      if (sleeping) {                                           \
        sleeping = false;                                       \
        cerr << "restarting ..." << endl;                       \
      }                                                         \
    }                                                           \
  }                                                             \
                                                                \
  if (gv_nb_processes_flag) {                                   \
                                                                \
    int lines = 0;                                              \
    FILE * f = popen("ps -A", "r");                             \
    int c = getc(f);                                            \
    while(c != EOF) {                                           \
      if (c == '\n')                                            \
        lines++;                                                \
      c = getc(f);                                              \
    }                                                           \
    pclose(f);                                                  \
                                                                \
    if (lines >= gv_nb_processes) {                             \
      cerr << "nb processes : " << lines << endl;               \
                                                                \
      if (!sleeping) {                                          \
        /* erase automata */                                    \
        for (int i=0; i < gv_nbseeds; i ++) {                   \
           if (a_s[i] != NULL){                                 \
             delete a_s[i];                                     \
             a_s[i] = NULL;                                     \
           }                                                    \
        }                                                       \
                                                                \
        /* security input */                                    \
        if (gv_nb_input_filenames > 0) {                        \
          for(int i=0;i<gv_nb_input_filenames;i++)              \
            inputPareto(l,gv_input_filenames[i]);               \
        }                                                       \
                                                                \
        /* final output */                                      \
        cerr << "# seeds\t sel\t sens\t dist"<<  endl;          \
        selectPareto(l);                                        \
        if (!gv_output_filename)                                \
          list_and_areaPareto(l);                               \
        else                                                    \
          outputPareto(l,gv_output_filename);                   \
        cerr << "sleeping ..." << endl;                         \
                                                                \
        sleeping = true;                                        \
      }                                                         \
      SLEEP(CSLEEP);                                            \
      goto restart;                                             \
    } else {                                                    \
      if (sleeping) {                                           \
        sleeping = false;                                       \
        cerr << "restarting ..." << endl;                       \
      }                                                         \
    }                                                           \
  }                                                             \
 }                                                              \
}                                                                     


// @}
#endif  

